<?php
include("setting.php");
$cnx = connectdb();
  
if (strcmp($_SERVER['REQUEST_METHOD'],'POST')) {
	echo 'Not a post request: '.$_SERVER['REQUEST_METHOD'];
	return;
}

$subject_code = $_POST['subjectCode'];
$study_id = $_POST['studyID'];
$module_nr = $_POST['moduleNr'];
$group_nr = $_POST['groupNr'];
$log_upload_cd = 4;

// check parameters - do not continue if missing
if (!$subject_code || !$study_id || !$module_nr || !$group_nr) {
  echo "Error: Missing information";
  exit;
}

// check whether the subject is allowed to upload data to this module of tatool online
$q_select = $cnx->prepare("SELECT SUB.SUBJECT_ID, MDL.MODULE_ID, GRP.GROUP_ID"
." FROM TAT_SUBJECT SUB, TAT_MODULE MDL, TAT_GROUP GRP, TAT_SUBJECT_GROUP SUBGRP, TAT_GROUP_MODULE GRPMDL, TAT_MODULE_STUDY MDLSTD "
." WHERE SUB.SUBJECT_ID = SUBGRP.SUBJECT_ID "
." AND SUBGRP.GROUP_ID = GRPMDL.GROUP_ID "
." AND GRPMDL.GROUP_ID = GRP.GROUP_ID "
." AND GRPMDL.MODULE_ID = MDLSTD.MODULE_ID "
." AND MDLSTD.MODULE_ID = MDL.MODULE_ID "
." AND SUB.STUDY_ID = :study_id "
." AND SUB.SUBJECT_CODE = :subject_code "
." AND MDL.MODULE_NR = :module_nr "
." AND MDLSTD.STUDY_ID = :study_id");
$q_select->bindParam(':study_id', $study_id);
$q_select->bindParam(':subject_code', $subject_code);
$q_select->bindParam(':module_nr', $module_nr);
$q_select->execute();

if ($q_select->rowCount() < 1) {
  echo "Error: Wrong code or the module you're looking for is not available";
  exit;
}

while ($field = $q_select->fetch(PDO::FETCH_ASSOC)) {
  $subject_id = $field['SUBJECT_ID'];
  $module_id = $field['MODULE_ID'];
  $group_id = $field['GROUP_ID'];
}

// set the upload path
$filepath=$uploadpath 
."studies".$separator.$study_id
.$separator."modules"
.$separator.$module_nr
.$separator."groups"
.$separator.$group_nr
.$separator."upload"
.$separator.$subject_code
.$separator;

// check if the directory already exists, create otherwise
if (! file_exists($filepath)) {
	if (! mkdir($filepath, 0700)) {
		die('Unable to create directory: '.$storagedir);
	}
} else {
	// make sure this represents a directory
	if (! is_dir($filepath)) {
		// TODO: cleanup
		die('Path does not represent a directory: '.$storagedir);
	}
}

// create a date prefix used for all uploaded filenames.
$prefix = date('YmdHis');
$suffix = ".csv";

// move all files to that directory, using the prefix and suffix for all files
$uploadCount = 0;
$filenames = array();
foreach ($_FILES as $file) {
    if ($file['error'] == UPLOAD_ERR_OK) {
        $tmp_name = $file["tmp_name"];
        $name = $prefix . "_" . $file["name"] . $suffix;  
        $filenames[$file["name"]] = "$name";
        if (move_uploaded_file($tmp_name, "$filepath$name")) {
			echo "Stored file in $filepath$name\n";
			$uploadCount++;
		} else {
			echo "Unable to move file to $filepath$name\n";
		}
    }
}

// add a log entry for the upload
addLogEntry($study_id, $group_id, $subject_id, $module_id, $log_upload_cd);

/**
 * Session Data Upload
 */ 
$row = 0;
$handle = fopen ($filepath . $filenames['sessionData'], "r");
$sess_id = 0;
$last_sess_id = 0;
$session_id = 0;
while ( ($data = fgetcsv ($handle, 1000, ";")) !== FALSE ) {
  $num = count ($data);
  $row++;
  
  // skip header
  if ($row > 1) {
    $sess_id = $data[2];
    if ($sess_id != $last_sess_id) {
    
      // check whether session already exists
    	$session_info = getSessionInfo($module_id, $group_id, $subject_id, $data[2], $data[4]);
    	
    	// add new session
      if (!$session_info) {
        $session_id = addSession($module_id, $group_id, $subject_id, $data, $prefix);
      }
    }
    
    // add session data
    if (!$session_info) {
      addSessionData($session_id, $data);
    }
    
    $last_sess_id = $sess_id;
  }
}
fclose ($handle); 
unlink($filepath . $filenames['sessionData']);


/**
 * Module Data Upload
 */ 
$row = 0;
$handle = fopen ($filepath . $filenames['moduleData'], "r");
while ( ($data = fgetcsv ($handle, 1000, ";")) !== FALSE ) {
  $num = count ($data);
  $row++;
  
  // skip header
  if ($row > 1) {
    // insert/update training data
    addUpdateModuleData($module_id, $group_id, $subject_id, $data);    
  }

}

fclose ($handle); 
unlink($filepath . $filenames['moduleData']);


/**
 * Account Data Upload
 */ 
$row = 0;
$handle = fopen ($filepath . $filenames['accountData'], "r");
while ( ($data = fgetcsv ($handle, 1000, ";")) !== FALSE ) {
  $num = count ($data);
  $row++;
  
  // skip header
  if ($row > 1) {
    // insert/update training data
    addUpdateAccountData($subject_id, $data);    
  }

}

fclose ($handle); 
unlink($filepath . $filenames['accountData']);

// done
echo "\nSUCCESS - $uploadCount files received";

function clean_filename($filename){//function to clean a filename string so it is a valid filename
  $reserved = preg_quote('\/:*?"<>|', '/');//characters that are  illegal on any of the 3 major OS's
  //replaces all characters up through space and all past ~ along with the above reserved characters
  return preg_replace("/([\\x00-\\x20\\x7f-\\xff{$reserved}])/e", "_", $filename);
}